#ifndef dbx_utilsH
#define dbx_utilsH
#include <string>
#include <list>
#include <vector>
using namespace std;

// dbx_EnsureStores: call this function to build up the global
// variable 'dbx_Stores', which is a vector containing all the
// Outlook Express identities on this machine.
void dbx_EnsureStores();

// dbx_EnsureFolders: call this to build up the global
// variable 'dbx_Folders', a vector containing all the
// folders for a given OE idententy
void dbx_EnsureFolders(const string name,const string ver);

// These functions test whether a file is a valid dbx file
bool IsValidDbxFolderFile(const string fn);
bool IsValidDbxMessageFile(const string fn);



enum TDbxFolderType {dftStructure, dftInbox, dftOutbox, dftSent, dftDeleted, dftDrafts, dftMail};

typedef struct
{ string id;   // e.g. {CAA16-...}
  string name; // the username, e.g. "Main Identity"
  string ver;  // the version of Outlook Express, e.g. "5.0"
  string path; // the path to the store root, e.g. "c:\\store"
} TDbxStore;


typedef struct
{ string name;    // e.g. "Joseph"
  string path;    // e.g. "Friends"
  string account; // e.g. "Cambridge email";
  string full;    // e.g. "Cambridge Email\\Friends\\Joseph"
  string fn;      // e.g. "cambridge email - inbox.dbx"
  string fpath;   // e.g. "c:\\localmessagestore"
  unsigned int depth; // e.g. 2 (Cam.Email=0, Friends=1, Joseph=2)
  TDbxFolderType type; // see the above enumeration
  bool might_contain_offlines;  // just a hint
  DWORD filesize; // this gets set at a later time, during processing
} TDbxFolderInfo;
bool operator <(const TDbxFolderInfo &a,const TDbxFolderInfo &b);


extern vector<TDbxStore> dbx_Stores;
extern vector<TDbxFolderInfo> dbx_Folders;




// The following dbx-reading code (and in the accompanying oe_utils.cpp)
// is written by Arne Shloh, available at his web page http://oe5dbx.aroh.de/
// Lucian has modified it to use memory-mapped files

typedef unsigned char   int1;
typedef unsigned short  int2;
typedef unsigned long   int4;

class imemfile
{ public:
  const char *buf; DWORD size;
  imemfile(const char *fn);
  ~imemfile();
  void close();
  void seekg(unsigned int apos);
  void read(char *dst, unsigned int size);
  bool okay();
protected:
  unsigned int pos;
  bool isokay;
  HANDLE hf;
  HANDLE hmap;
};



class DbxException
{ public:
    DbxException(const std::string &text) { Error = text; }
    const char * what() const { return Error.c_str(); }
  private:
    std::string Error;
};


class DbxTree
{ public:
    DbxTree(imemfile &inm, int4 address, int4 values);
    ~DbxTree();
    int4 GetValue(int4 index) const;
  private:
    void readValues(imemfile &inm, int4 parent, int4 address, int4 position, int4 values);
    int4 Address; int4 Values, *Array;
};

const int4 FileHeaderSize    = 0x24bc;
const int4 FileHeaderEntries = FileHeaderSize>>2;
const int4 fhFileInfoLength       = 0x07;
const int4 fhFirstFolderListNode  = 0x1b, fhLastFolderListNode  = 0x1c;
const int4 fhMessageConditionsPtr = 0x22, fhFolderConditionsPtr = 0x23;
const int4 fhEntries              = 0x31, fhTreeRootNodePtr     = 0x39 ;

class DbxFileHeader
{ public:
    DbxFileHeader(imemfile &inm);
    int4 GetValue(int4 index) const{return Buffer[index];    }
    bool isFolders() const { return (Buffer && (Buffer[1]==0x6f74fdc6)); }
  private:
    void readFileHeader(imemfile &inm); // this function is called from the constructor
    int4 *Buffer;                       // stores the data
};

class DbxFileInfo
{ public:
    DbxFileInfo(imemfile &inm, int4 address, int4 length);
    ~DbxFileInfo();
    const char * GetFolderName()   const;
    std::string  GetFileInfoTime() const;
  private:
    void init();
    void readFileInfo(imemfile &inm);
    //data
    int4 Address, Length; int1 *buffer;
};


const int1 DbxFolderListSize    = 0x68;
const int1 DbxFolderListEntries = DbxFolderListSize >> 2;
const int1 flFolderInfo         = 0x15;
const int1 flNextFolderListNode = 0x17, flPreviousFolderListNode = 0x18;



class DbxMessage
{ public:
    DbxMessage(imemfile &inm, int4 address);
    ~DbxMessage();
    int4 GetLength() const { return Length; }
    char * GetText() const { return Text; } 
    void Analyze(int4 & headerLines, int4 & bodyLines) const;
  private:
    void init();
    void readMessageText(imemfile &inm);
    int4 Address, Length; char * Text; // address, length, text of message
};


// The data types I found stored in the indexed info objects
enum IndexedInfoDataType { dtNone=0, dtInt4=1, dtDateTime=2, dtString=4, dtData=8 };
//
const int1 MaxIndex = 0x7F;

class DbxIndexedInfo
{ public:
    DbxIndexedInfo(imemfile &inm, int4 address);
    ~DbxIndexedInfo();
    int4 GetAddress()    const { return Address;    }
    int4 GetBodyLength() const { return BodyLength; }
    int1 GetEntries()    const { return Entries;    }
    int1 GetCounter()    const { return Counter;    }
    int4 GetIndexes()    const { return Indexes;    }
    virtual const char * GetIndexText(int1 index) const {return 0;}
    virtual IndexedInfoDataType GetIndexDataType(int1 index) const {return dtNone;}
    int1 * GetValue(int1 index, int4 * length) const;
    int4 GetValue(int1 index)                const;
  private:
    void init();
    void readIndexedInfo(imemfile &inm);
    //
    void SetIndex(int1 index, int1 * begin, int4 length=0);
    void SetEnd(int1 index, int1 * end);
    // message info data
    int4 Address, BodyLength;
    int2 ObjectLength;
    int1 Entries, Counter, * Buffer;
    int4 Indexes;            // Bit field for the used indexes (MaxIndexe bits)
    int1 * Begin[MaxIndex];
    int4 Length[MaxIndex];
};


const int1 fiiIndex = 0x00, fiiParentIndex = 0x01, fiiName = 0x02, fiiFlags = 0x06;

class DbxFolderInfo : public DbxIndexedInfo
{ public:
    DbxFolderInfo(imemfile &inm, int4 address) : DbxIndexedInfo(inm, address) { }
    const char * GetIndexText(int1 index) const;
    IndexedInfoDataType GetIndexDataType(int1 index) const;
};




const int1 miiIndex = 0x00, miiFlags = 0x01, miiMessageAddress = 0x04;

class DbxMessageInfo : public DbxIndexedInfo
{ public:
    DbxMessageInfo(imemfile &inm, int4 address) : DbxIndexedInfo(inm, address) { }
    const char *        GetIndexText    (int1 index) const;
    IndexedInfoDataType GetIndexDataType(int1 index) const;
};



#endif
